/*
 * simple_radio.c
 *
 * Created: 2016-09-28 08:57:54
 *  Author: dsosnowski
 */ 

//----------------------------------------------------
#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "simple_r.h"

//----------------------------------------------------
#define SIMPLER_PORT				PORTB
#define SIMPLER_DDR					DDRB
#define SIMPLER_IN					PINB
#define RECEIVE_DATA_PIN			2
#define RECEIVE_DATA_GET			(SIMPLER_IN & (1<<RECEIVE_DATA_PIN))

//----------------------------------------------------
#define SAMPLE_MAIN_FREQ			F_CPU
#define SAMPLE_DIVIDER				64
#define SAMPLE_FREQ					10000
#define SAMPLE_PERIOD				100		// = 1 / SAMPLE_FREQ
#define SAMPLE_PER_MS				10		// = 1ms / SAMPLE_PERIOD 
#define SAMPLE_TIMER_OVF			((SAMPLE_MAIN_FREQ / SAMPLE_DIVIDER) / SAMPLE_FREQ)

//----------------------------------------------------
#define SIMPLER_SHORT_TIME			500		//min 0,5ms	max 10ms
#define SIMPLER_LONG_TIME			1500
#define SIMPLER_FRAME2RAME			10000		//min 10ms

#define TRANSMIT_LO_STATE			(SIMPLER_SHORT_TIME / SAMPLE_PERIOD)
#define TRANSMIT_HI_STATE			(SIMPLER_LONG_TIME / SAMPLE_PERIOD)
#define TRANSMIT_PERIOD				((SIMPLER_LONG_TIME + SIMPLER_SHORT_TIME) / SAMPLE_PERIOD)
#define TRANSMIT_INIT_PULSE			(TRANSMIT_HI_STATE)
#define TRANSMIT_NORM_PULSE			(TRANSMIT_LO_STATE)
#define TRANSMIT_FRAME_DELAY		(SIMPLER_FRAME2RAME / SAMPLE_PERIOD)

#define RECEIVE_TOLERANCE			3
//#define RECEIVE_MAXLEN			(SIMPLER_BUFFER - 1)
#define RECEIVE_MAXLEN				32
#define RECEIVE_MINLEN				10
#define TRANSMIT_REPEAT_CYCLE		5
#define SIMPLER_BUFFER				64

//----------------------------------------------------
#define RECEIVE_EXPECTED_LEN		25
#define RECEIVER_STEP_SCAN			0
#define RECEIVER_STEP_CHECK			1
#define RECEIVER_STEP_COLLECT		2
#define RECEIVER_STEP_READY			3


struct {
	char buffer[SIMPLER_BUFFER];
	char index;
	volatile char rxlen;
	volatile char rxstatus;
} simpler;

struct {
	char address;
	char command;
} radiodata;

void SimplerByteCodec(char *dst, char data);
char SimplerByteDecodec(char *src);


//----------------------------------------------------
void SimplerInit(void) {
	SIMPLER_PORT  |= (1<<RECEIVE_DATA_PIN);
	SimplerTimerStart();
	sei();
}

//----------------------------------------------------
void SimplerTimerStart(void) {
	TCCR0A = (1<<WGM01 | 1<<WGM00);
	TCCR0B = (1<<WGM02 | 1<<CS01 | 1<<CS00);
	OCR0A = SAMPLE_TIMER_OVF;
	//TIMSK0 = (1<<OCIE0A);
	TIMSK0 = (1<<TOIE0);
}

//----------------------------------------------------
//ISR(TIMER0_COMPA_vect){
ISR(TIM0_OVF_vect){
	TCNT0 = 0;
	static char bittime;
	static char bitindex;
			
			if (simpler.rxstatus == RECEIVER_STEP_SCAN) {		//searching init bit
				if (RECEIVE_DATA_GET == 0) {
					if (bittime < 0xFF) bittime++;
					if (bittime > (TRANSMIT_PERIOD + TRANSMIT_PERIOD + TRANSMIT_PERIOD)) {		//long delay - frame end
						simpler.rxstatus = RECEIVER_STEP_CHECK;					
						bittime = 0;
						bitindex = 0;
					}
				} else {
					bittime = 0;					
					bitindex = 1;
				}
			}

			else if (simpler.rxstatus == RECEIVER_STEP_CHECK) {		//searching init bit
				if (RECEIVE_DATA_GET > 0) {
					if (bittime < 0xFF) bittime++;
					bitindex = 1;
				} else {
					if (bitindex == 1) {
						if ((bittime >= (TRANSMIT_INIT_PULSE - RECEIVE_TOLERANCE)) &&
							(bittime <= (TRANSMIT_INIT_PULSE + RECEIVE_TOLERANCE))) {
							simpler.rxstatus = RECEIVER_STEP_COLLECT;
							simpler.buffer[0] = bittime;
							simpler.index = 1;
						} else {
							simpler.rxstatus = RECEIVER_STEP_SCAN;							
						}					
					}
					bittime = 0;
					bitindex = 0;
				}
			}
			
			else if (simpler.rxstatus == RECEIVER_STEP_COLLECT) {		//collect frame
				//simpler.rxstatus = RECEIVER_STEP_SCAN;
				
				if (RECEIVE_DATA_GET > 0) {
					if (bitindex == 0) {
						bittime = 0;
					}
					if (bittime < 0xFF) bittime++;
					bitindex = 1;

				} else {
					if (bitindex == 1) {
						if (bittime >= (TRANSMIT_LO_STATE - RECEIVE_TOLERANCE)) {
							if (bittime <= (TRANSMIT_HI_STATE + RECEIVE_TOLERANCE)) {
								simpler.buffer[simpler.index] = bittime;				//collect new bit
								if (simpler.index < RECEIVE_MAXLEN) {
									simpler.index++;
								} else {
									simpler.rxstatus = RECEIVER_STEP_SCAN;
								}
							} else {
								simpler.rxstatus = RECEIVER_STEP_SCAN;
							}
						} else {
							simpler.rxstatus = RECEIVER_STEP_SCAN;
						}
						bittime = 0;	
					
					} else {
						if (bittime > (TRANSMIT_PERIOD + TRANSMIT_PERIOD + TRANSMIT_PERIOD)) {		//long delay - frame end
							if (simpler.index >= RECEIVE_MINLEN) {
								simpler.rxlen = simpler.index;
								simpler.rxstatus = RECEIVER_STEP_READY;
							} else {
								simpler.rxstatus = RECEIVER_STEP_SCAN;
							}
							bittime = 0;
						}						
					}

					if (bittime < 0xFF) bittime++;
					bitindex = 0;					
				}
			}
}

/*//----------------------------------------------------
char SimplerFramePrepare(char address, char command) {
	simpler.buffer[0] = TRANSMIT_HI_STATE;
	SimplerByteCodec(&simpler.buffer[1], address);
	SimplerByteCodec(&simpler.buffer[9], command);
	SimplerByteCodec(&simpler.buffer[17], command);
	simpler.buffer[25] = TRANSMIT_HI_STATE;		
	simpler.txlen = 25;
	return simpler.txlen;
}

//----------------------------------------------------
void SimplerByteCodec(char *dst, char data) {
	char i = 0;
	for(i = 0; i < 8; i++) {
		if ((data & (0x80 >> i)) == 0) {
			*dst = TRANSMIT_LO_STATE;
		} else {
			*dst = TRANSMIT_HI_STATE;
		}
		dst++;
	}
}*/

//----------------------------------------------------
char SimplerReceiveCheck(void) {
	if (simpler.rxstatus == RECEIVER_STEP_READY) {
		return 1;
	}
	return 0;
}

//----------------------------------------------------
char SimplerReceiveLength(void) {
	return simpler.rxlen;
}

//----------------------------------------------------
char SimplerDecode(void) {
	if (simpler.rxlen >= RECEIVE_EXPECTED_LEN) {
		radiodata.address =  SimplerByteDecodec(&simpler.buffer[9]);
		radiodata.command =  SimplerByteDecodec(&simpler.buffer[17]);		
		return 1;
	}
	return 0;
}

//----------------------------------------------------
char SimplerByteDecodec(char *src) {
	char data = 0;
	char i;
	for(i=0; i < 8; i++){
		data <<= 1;
		if (*src <= TRANSMIT_LO_STATE + RECEIVE_TOLERANCE) {
			data |= 0;
		} else {
			data |= 1;
		}
		src++;
	}
	return data;
}

//----------------------------------------------------
void SimplerReceiveClear(void) {
	simpler.rxlen = 0;
	simpler.rxstatus = 0;
}

//----------------------------------------------------
char *SimplerBufferPt(char index) {
	return &simpler.buffer[index];
}

//----------------------------------------------------
char SimplerBufferGet(char index) {
	return simpler.buffer[index];
}

//----------------------------------------------------
char RadioDataGet(char i) {
	if (i == 0) return radiodata.address;
	if (i == 1) return radiodata.command;
	return 0;
}
