/*
 * ui.c
 *
 * Created: 03.03.2017 14:23:24
 *  Author: dsosnowski
 */ 

//----------------------------------------------------
#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "ui.h"
#include "defs.h"

#define LED_PORT			PORTA
#define LED_DDR				DDRA
#define LED_PIN				2
#define LED_SET				(LED_PORT |= (1<<LED_PIN))
#define LED_OFF				(LED_PORT &= ~(1<<LED_PIN))

#define REL_PORT			PORTA
#define REL_DDR				DDRA
#define REL_PIN				0
#define REL_SET				(REL_PORT |= (1<<REL_PIN))
#define REL_OFF				(REL_PORT &= ~(1<<REL_PIN))

#define SW_PORT				PORTA
#define SW_IN				PINA
#define SW_PIN				6
#define SW_GET				(SW_IN & (1<<SW_PIN))

#define REG_PORT			PORTA
#define REG_IN				PINA
#define REG_PIN				1
#define REG_CH				1

#define CFG_PORT			PORTA
#define CFG_IN				PINA
#define CFG_CONFIG_PIN		5
#define CFG_MODE_PIN		4
#define CFG_TIME_PIN		3

#define BUTTONS_CLICK_THR	3
#define BUTTONS_HOLD_THR	200


//----------------------------------------------------
struct {
	uint8_t states;
	uint8_t actions;
	uint8_t led;
} ui;


struct {
	uint8_t state;
	uint8_t action;
	uint8_t hold;
} output;

//----------------------------------------------------
void UiInit(void){
	LED_PORT |= (1<<LED_PIN | 1<<CFG_CONFIG_PIN | 1<<CFG_MODE_PIN | 1<<CFG_TIME_PIN);
	LED_DDR |= (1<<LED_PIN | 1<<REL_PIN);	
	AnalogInit();
}

//----------------------------------------------------
void UiLed(uint8_t state){
	if (state == 0) 
		LED_OFF;
	else
		LED_SET;
}

//----------------------------------------------------
void UiRelay(uint8_t state){
	if (state == 0)
		REL_OFF;
	else
		REL_SET;
}

//----------------------------------------------------------
void UiOutputAction(uint8_t action){
	output.action = action;
}

//-------------------------------------------------------
void UiOutputManager(void){
	static uint16_t countdown;
	static uint8_t persec;
	
	
	if (UiModeGet() == 0) {			//bistable
		if (output.hold > 0) output.hold--;
		
		if (output.action > 0){
			
			if (output.hold == 0) {
				if (output.state == 0){
					output.state = 1;	
					countdown = UiRegGet();
				} else {
					output.state = 0;
				}
			}
			
			output.hold = OUT_BI_HOLD;
			output.action = 0;
		}
		
	
		persec++;
		if (persec > (1000 / MAIN_DELAY)) {
			persec = 0;
			if (countdown > 0) {
				countdown--;
				if (countdown == 0) {
					if (UiTimeGet() > 0){
						output.state = 0;
					}
				}
			}
		}		
		
	} else {						//monostable
		if (output.hold > 0) {
			output.hold--;
			if (output.hold == 0){
				output.state = 0;
			}
		}
		
		if (output.action > 0){
			output.state = 1;
			output.hold = OUT_MONO_HOLD;
			output.action = 0;
		}
	}
	
	if (output.state > 0) UiRelay(1);
	else UiRelay(0);
}

//----------------------------------------------------------
uint8_t UiOutputGet(void){
	return output.state;
}

//----------------------------------------------------------
uint8_t UiOutputHoldGet(void){
	if (output.hold > 0) return 1;
	return 0;
}

//----------------------------------------------------
uint8_t UiButtonProc(void){
	uint8_t state;
	uint8_t result = 0;

		state = SW_GET;
		if (state == 0){
			if (ui.states < 0xFF) {
				ui.states++;
				if (ui.states == BUTTONS_CLICK_THR) result = BUTTON_CLICK;
				if (ui.states == BUTTONS_HOLD_THR) result = (BUTTON_CLICK + BUTTON_HOLD);
			}
		} else {
			ui.states = 0;
		}
	
	ui.actions = result;	
	return result;	
}

//----------------------------------------------------
uint8_t UiButtonGet(void){
	if (ui.states >= BUTTONS_CLICK_THR) return BUTTON_CLICK;
	return 0;
}

//----------------------------------------------------
uint8_t UiModeGet(void){
	if ((CFG_IN & (1<<CFG_MODE_PIN)) == 0){
		return 1;
	}
	return 0;
}

//----------------------------------------------------
uint8_t UiTimeGet(void){
	if ((CFG_IN & (1<<CFG_TIME_PIN)) == 0){
		return 1;
	}
	return 0;
}

//----------------------------------------------------
uint8_t UiConfigGet(void){
	if ((CFG_IN & (1<<CFG_CONFIG_PIN)) == 0){
		return 1;
	}
	return 0;
}

//----------------------------------
void AnalogInit(void){
	DIDR0 = (1<<REG_CH);
	ADCSRB = (1<<ADLAR);
	ADCSRA = (1<<ADEN | 1<<ADATE | 1<<ADPS2 | 1<<ADPS1);
	ADMUX = (REG_CH);
	ADCSRA |= (1<<ADSC);
}

//----------------------------------------------------
uint16_t UiRegGet(void){
	uint8_t val;
	val = ADCH;
	if (val == 0) val = 1;
	return (val << 1);
}
