/*
 * simple_radio.c
 *
 * Created: 2016-09-28 08:57:54
 *  Author: dsosnowski
 */ 

//----------------------------------------------------
#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "simple_r.h"
#include "ui.h"


//----------------------------------------------------
#define SIMPLER_PORT				PORTB
#define SIMPLER_DDR					DDRB
#define SIMPLER_IN					PINB
//#define RECEIVE_DATA_PIN			2
//#define RECEIVE_DATA_GET			(SIMPLER_IN & (1<<RECEIVE_DATA_PIN))
#define TRANSMIT_DATA_PIN			2
#define TRANSMIT_DATA_SET			(SIMPLER_PORT |= (1<<TRANSMIT_DATA_PIN))
#define TRANSMIT_DATA_OFF			(SIMPLER_PORT &= ~(1<<TRANSMIT_DATA_PIN))

//----------------------------------------------------
#define SAMPLE_MAIN_FREQ			F_CPU
#define SAMPLE_DIVIDER				64
#define SAMPLE_FREQ					10000
#define SAMPLE_PERIOD				100		// = 1 / SAMPLE_FREQ
#define SAMPLE_PER_MS				10		// = 1ms / SAMPLE_PERIOD 
#define SAMPLE_TIMER_OVF			((SAMPLE_MAIN_FREQ / SAMPLE_DIVIDER) / SAMPLE_FREQ)

//----------------------------------------------------
#define SIMPLER_SHORT_TIME			500		//min 0,5ms	max 10ms
#define SIMPLER_LONG_TIME			1500
#define SIMPLER_FRAME2RAME			10000	//min 10ms

#define TRANSMIT_LO_STATE			(SIMPLER_SHORT_TIME / SAMPLE_PERIOD)
#define TRANSMIT_HI_STATE			(SIMPLER_LONG_TIME / SAMPLE_PERIOD)
#define TRANSMIT_PERIOD				((SIMPLER_LONG_TIME + SIMPLER_SHORT_TIME) / SAMPLE_PERIOD)
#define TRANSMIT_INIT_PULSE			(TRANSMIT_HI_STATE)
#define TRANSMIT_NORM_PULSE			(TRANSMIT_LO_STATE)
#define TRANSMIT_FRAME_DELAY		(SIMPLER_FRAME2RAME / SAMPLE_PERIOD)

#define RECEIVE_TOLERANCE			3
//#define RECEIVE_MAXLEN			(SIMPLER_BUFFER - 1)
#define RECEIVE_MAXLEN				32
#define RECEIVE_MINLEN				10
#define TRANSMIT_REPEAT_CYCLE		5
#define SIMPLER_BUFFER				32

//----------------------------------------------------
#define RECEIVE_EXPECTED_LEN		25
#define RECEIVER_STEP_SCAN			0
#define RECEIVER_STEP_CHECK			1
#define RECEIVER_STEP_COLLECT		2
#define RECEIVER_STEP_READY			3


struct {
	char buffer[SIMPLER_BUFFER];
	char index;
	volatile char txlen;
	volatile char txrun;
	volatile char txstatus;
	//volatile char rxlen;
	//volatile char rxstatus;
} simpler;

struct {
	char address;
	char command;
} radiodata;

void SimplerByteCodec(char *dst, char data);
char SimplerByteDecodec(char *src);


//----------------------------------------------------
void SimplerInit(void) {
	SIMPLER_DDR  |= (1<<TRANSMIT_DATA_PIN);
	SimplerTimerStart();
	sei();
}

//----------------------------------------------------
void SimplerTimerStart(void) {
	TCCR0A = (1<<WGM01 | 1<<WGM00);
	TCCR0B = (1<<WGM02 | 1<<CS01 | 1<<CS00);
	OCR0A = SAMPLE_TIMER_OVF;
	//TIMSK0 = (1<<OCIE0A);
	TIMSK = (1<<TOIE0);
}

//----------------------------------------------------
//ISR(TIMER0_COMPA_vect){
ISR(TIMER0_OVF_vect){
	TCNT0 = 0;
	static char bittime;
	static char bitindex;
	
	if (simpler.txstatus > 0) {
		
		if (simpler.index < simpler.txlen) {		//wait for bit countdown
			if (bittime > 0) {
				bittime--;
			} else {
				if ((bitindex & 1) == 0) {
					TRANSMIT_DATA_SET;
					if (simpler.index == 0) {		//init bit
						UiLed(1);
						bittime = TRANSMIT_INIT_PULSE;
					} else {						//normal bits
						//bittime = TRANSMIT_NORM_PULSE;
						bittime = simpler.buffer[simpler.index];
					}
					bitindex = 1;
				} else {
					TRANSMIT_DATA_OFF;
					if (simpler.index == (simpler.txlen - 1)) {		//last delay
						UiLed(0);
						bittime = TRANSMIT_FRAME_DELAY;
					} else {
						//bittime = simpler.buffer[simpler.index];	//bits delay
						bittime = (TRANSMIT_PERIOD - simpler.buffer[simpler.index]);
					}
					bitindex = 0;
					simpler.index++;
				}
			}
		} else {
			if (bittime > 0) {					//last bit
				bittime--;
			} else {
				simpler.index = 0;
				bitindex = 0;

				if (simpler.txstatus == 1) simpler.txrun = 0;	//after firs frame automatically clear run flag 
				
				if (simpler.txstatus <= TRANSMIT_REPEAT_CYCLE) {
					simpler.txstatus++;					
				} else {					
					if (simpler.txrun > 0) {		//repeat
						if (simpler.txstatus < 0xFF) simpler.txstatus++;
					} else {						//or exit
						simpler.txstatus = 0;
						simpler.txlen = 0;
					}
				}
			}
		}
			
	} else {				
		if (simpler.txrun > 0) {				//new tx
			if (simpler.txlen > 0) {
				simpler.txstatus = 1;
				simpler.index = 0;
				bitindex = 0;
				bittime = 0;
			}
		}	
	}
}

//----------------------------------------------------
char SimplerFramePrepare(char address, char command) {
	simpler.buffer[0] = TRANSMIT_HI_STATE;
	SimplerByteCodec(&simpler.buffer[1], 0x55);
	SimplerByteCodec(&simpler.buffer[9], address);
	SimplerByteCodec(&simpler.buffer[17], command);
	simpler.buffer[25] = TRANSMIT_HI_STATE;		
	simpler.txlen = 25;
	return simpler.txlen;
}

//----------------------------------------------------
void SimplerByteCodec(char *dst, char data) {
	char i = 0;
	for(i = 0; i < 8; i++) {
		if ((data & (0x80 >> i)) == 0) {
			*dst = TRANSMIT_LO_STATE;
		} else {
			*dst = TRANSMIT_HI_STATE;
		}
		dst++;
	}
}

//----------------------------------------------------
void SimplerTransmitRun(char run) {
	if (run > 0){
		if (simpler.txlen > 0) {
			simpler.txrun = 1;	
		}
	} else {
		simpler.txrun = 0;
	}
}

//----------------------------------------------------
char SimplerTransmitCheck(void) {
	return simpler.txstatus;
}
